package runtime_test

import (
	"internal/testenv"
	"os"
	"path/filepath"
	"strings"
	"testing"
)

func TestSyscalltrace_StaticUpToDate(t *testing.T) {
	testenv.MustHaveGoBuild(t)

	// This test checks that the staticSyscalls array in runtime/zsyscalltrace_windows.go
	// is up-to-date with the list of syscalls in the runtime. It does this by
	// running syscalltrace_generate_windows.go to generate the list of syscalls
	// and comparing it to the list in runtime/zsyscalltrace_windows.go.

	tmp := t.TempDir()
	zout := filepath.Join(tmp, "tmp.go")
	cmd := testenv.Command(t, testenv.GoToolPath(t), "run", "syscalltrace_generate_windows.go", "-output", zout, "os_windows", "syscalltrace_windows")
	out, err := cmd.CombinedOutput()
	if err != nil {
		t.Fatalf("generating syscalls: %v\n%s", err, out)
	}
	want, err := os.ReadFile(zout)
	if err != nil {
		t.Fatal(err)
	}
	got, err := os.ReadFile(filepath.Join(testenv.GOROOT(t), "src", "runtime", "zsyscalltrace_windows.go"))
	if err != nil {
		t.Fatal(err)
	}
	if string(got) != string(want) {
		t.Errorf("syscalltrace_windows.go is out of date. Run go generate in runtime/syscalltrace_windows.go to update it.")
	}
}

func TestSyscalltrace(t *testing.T) {
	testenv.MustHaveGoBuild(t)

	tmp := t.TempDir()
	source := filepath.Join(tmp, "tmp.go")
	bin := filepath.Join(tmp, "tmp.exe")
	trace := filepath.Join(tmp, "trace.csv")
	err := os.WriteFile(source, []byte(`
		package main
		import "syscall"
		func main() {
			syscall.CloseHandle(0)
		}
	`), 0644)
	if err != nil {
		t.Fatal(err)
	}
	cmd := testenv.Command(t, testenv.GoToolPath(t), "build", "-o", bin, source)
	out, err := cmd.CombinedOutput()
	if err != nil {
		t.Fatalf("building test program: %v\n%s", err, out)
	}
	cmd = testenv.Command(t, bin)
	cmd.Env = []string{"MS_GO_SYSCALL_TRACE=" + trace}
	out, err = cmd.CombinedOutput()
	if err != nil {
		t.Fatalf("running test program: %v\n%s", err, out)
	}
	if _, err := os.Stat(trace); err != nil {
		t.Fatalf("trace file not written: %v", err)
	}
	csv, err := os.ReadFile(trace)
	if err != nil {
		t.Fatalf("reading trace file: %v", err)
	}
	if !strings.Contains(string(csv), "KERNEL32.DLL,CloseHandle\n") {
		t.Errorf("trace file does not contain expected syscall")
	}
}
